 
 /*------------------------------------------------------------------------
    File        : AuthCP
    Purpose     : Maintain a secured client identity.
    Syntax      : 
    Description : 
    Author(s)   : clongo
    Created     : Fri Apr 04 12:56:38 EDT 2008
    Notes       : 
  ----------------------------------------------------------------------*/



CLASS AuthCP   : 

    DEFINE PRIVATE PROPERTY rEncryptKey AS RAW
      GET:
          RETURN GENERATE-PBE-KEY('plover').
      END.
      
    DEFINE PRIVATE PROPERTY cDomainAccessKey AS CHAR
      GET:
         RETURN "bluebird". /* Setup via Data Admin Tool--Authentication System Domain. */
      END.


	CONSTRUCTOR PUBLIC  AuthCP (  ):
		SUPER ().
	END CONSTRUCTOR.

    

	METHOD PUBLIC LOGICAL createCPObject (INPUT pcUserID AS CHAR,
                                          INPUT pcPasswd AS CHAR,
                                          OUTPUT pcMessage AS CHARACTER,
                                          OUTPUT pcSecToken AS CHARACTER,
                                          OUTPUT prCP AS RAW
                                          ):
        DEFINE BUFFER bCPObject FOR CPObject.
                                              
        DEFINE VAR hClientPrincipal AS HANDLE NO-UNDO.
        
        DEFINE VAR cSecToken AS CHAR NO-UNDO.
        DEFINE VARIABLE lStatus AS LOGICAL NO-UNDO.
        
        FIND Identity WHERE Identity.identityName = pcUserID NO-LOCK.
        
        

        /* Create Client Principal Object */
        CREATE CLIENT-PRINCIPAL hClientPrincipal.
        
        /* Set CP Object Values */
        hClientPrincipal:SESSION-ID = BASE64-ENCODE(GENERATE-UUID).
        hClientPrincipal:USER-ID = pcUserID.
        hClientPrincipal:DOMAIN-NAME = 'bravepoint.com'.
        hClientPrincipal:DOMAIN-TYPE = 'Internal'.
        hClientPrincipal:LOGIN-EXPIRATION-TIMESTAMP = ADD-INTERVAL(NOW, 60, 'seconds').
       
        
        
        IF Identity.IdentityKey = ENCODE(pcPasswd) THEN DO:
          /* Lock Down the CP Object. Note: DomainAccessKey should come from
           a secure location. */
           
          
           hClientPrincipal:ROLES = Identity.identityRoles.
           hClientPrincipal:SET-PROPERTY('UserPlant', 'Norcross'). 

          IF NOT hClientPrincipal:SEAL(cDomainAccessKey) THEN 
            UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE('CP Object Seal Failed. login-state = &1', hClientPrincipal:LOGIN-STATE)).
        
          /* Set UserID accross all connected DBs. The UserID within the 
           CP Object now becomes the Session wide UserID */
          IF NOT SET-DB-CLIENT(hClientPrincipal) THEN DO:
            hClientPrincipal:LOGOUT().
            UNDO, THROW NEW Progress.Lang.AppError('Could not set Userid for connected DBs.', 106).
          END.
          lStatus = TRUE.
        END.
        ELSE DO:
           /* This will set the state-detail attribute */ 
           hClientPrincipal:AUTHENTICATION-FAILED('UserName Password authenitication failed.').
           pcMessage =  'UserName Password authenitication failed.'.
        END.
        
        
       
        MESSAGE "createCPObject....Status:" 
           SKIP 'Login State: ' hClientPrincipal:LOGIN-STATE 
           SKIP "UserID Sports2000: " USERID("sports2000") 
           SKIP "UserID Security: " USERID("security") 
           SKIP "CP UserID:" hClientPrincipal:USER-ID VIEW-AS ALERT-BOX.
             
        /* Create a Identity Token to Pass to Client or other Agents */
        cSecToken = hClientPrincipal:SESSION-ID.
         
        /* Encrypt the token */
        pcSecToken = BASE64-ENCODE(ENCRYPT(cSecToken, rEncryptKey)).
         
         
        
        /* Store the CP Object as part of a user's session context. */
        prCP = hClientPrincipal:EXPORT-PRINCIPAL().
        DO TRANSACTION:
           CREATE bCPObject.
           ASSIGN
             bCPObject.SessionID = hClientPrincipal:SESSION-ID
             bCPObject.ContextObject = prCP.
        END.
             
       
        
        
                                          
        RETURN lStatus EQ TRUE.    
        
        CATCH eSysError AS Progress.Lang.Error :
            
            UNDO, THROW eSysError.
        		
        END CATCH.   
        FINALLY:
          IF VALID-HANDLE(hClientPrincipal) THEN
            DELETE OBJECT hClientPrincipal NO-ERROR.
        END.                        
                                          
    END. /* createCPObject */

    METHOD PUBLIC LOGICAL AuthenticateClient (INPUT pcSecToken AS CHARACTER,
                                              OUTPUT prCP AS RAW):

        DEFINE BUFFER bCPObject FOR CPObject.
        
        DEFINE VARIABLE hClientPrincipal AS HANDLE NO-UNDO.
        DEFINE VARIABLE cSessionID AS CHAR NO-UNDO.
        DEFINE VARIABLE lStatus AS LOGICAL NO-UNDO.
        
        
        DEFINE VARIABLE rKey AS RAW NO-UNDO.
        
        
        /* Dycrypt the Security Token. The info in the token is used to 
           re-establish a user's session context. */        
        cSessionID = STRING(DECRYPT(BASE64-DECODE(pcSecToken), rEncryptKey)) NO-ERROR.
        
                
        IF cSessionID = "" OR cSessionID = ? THEN
          UNDO, THROW NEW Progress.Lang.AppError('Unable to authenticate user. Could not dycrypt security token.', 102).
        
        
       
        
        
        
        /* Create an empty CP Object. */
        CREATE CLIENT-PRINCIPAL hClientPrincipal.
        
        /* Find the session context row containing the previosuly saved
           CP Object Data. */
        FIND bCPObject WHERE bCPObject.SessionID = cSessionID NO-LOCK NO-ERROR.
        IF NOT AVAIL bCPObject THEN
          UNDO, THROW NEW Progress.Lang.AppError('Unable to authenticate user. Could not find CPObject context.', 104).
           
        
        /* Load the CP Object. So you left with a CP Object as it
           existed after you sealed it during createCPObject. */
        hClientPrincipal:IMPORT-PRINCIPAL(bCPObject.ContextObject).
        
        
        
        IF NOT hClientPrincipal:VALIDATE-SEAL(cDomainAccessKey) THEN DO:
            
           hClientPrincipal:LOGOUT().
           
           DO TRANSACTION:
               FIND CURRENT bCPObject EXCLUSIVE-LOCK.
               prCP = hClientPrincipal:EXPORT-PRINCIPAL().              
               bCPObject.ContextObject = prCP.
           END. 
            
           UNDO, THROW NEW Progress.Lang.AppError(
           SUBSTITUTE('CP Object Validation Failed. Login-State = &1', hClientPrincipal:LOGIN-STATE), 105).
        END.
          
        /* Check expiration */  
        IF hClientPrincipal:LOGIN-EXPIRATION-TIMESTAMP < NOW THEN DO:
           /* This will set the state-detail attribute */ 
           hClientPrincipal:AUTHENTICATION-FAILED('User Session Expired.').
           
           hClientPrincipal:LOGOUT().
           
           DO TRANSACTION:
               FIND CURRENT bCPObject EXCLUSIVE-LOCK.
               
               prCP = hClientPrincipal:EXPORT-PRINCIPAL().
               bCPObject.ContextObject = prCP.
           END. 
           
           UNDO, THROW NEW Progress.Lang.AppError('User Session Expired.', 106).
           
        END.    
        
        prCP = hClientPrincipal:EXPORT-PRINCIPAL().
        
        IF SET-DB-CLIENT(hClientPrincipal) THEN DO:
          IF hClientPrincipal:LOGIN-STATE = "login" THEN 
             MESSAGE "AuthenticateClient....You are golden:" 
                   SKIP "UserID Sports2000: " USERID("sports2000") 
                   SKIP "UserID Security: " USERID("Security") 
                   SKIP "CP UserID:" hClientPrincipal:USER-ID 
                   SKIP "User Roles:" hClientPrincipal:ROLES VIEW-AS ALERT-BOX.
          ELSE
             MESSAGE "Login State: " hClientPrincipal:LOGIN-STATE VIEW-AS ALERT-BOX.
        END.
        ELSE
           UNDO, THROW NEW Progress.Lang.AppError('Unable to authenticate user. Could not set UserID', 107).
        
        
        lStatus = TRUE.
                                          
        RETURN lStatus EQ TRUE.    
        
        CATCH eSysError AS Progress.Lang.Error :
            
            
            UNDO, THROW eSysError.
                
        END CATCH.   
        FINALLY:
          IF VALID-HANDLE(hClientPrincipal) THEN
            DELETE OBJECT hClientPrincipal NO-ERROR.
        END.                      
        
    END.

	/*------------------------------------------------------------------------------
		Purpose:  
		Notes:  
	------------------------------------------------------------------------------*/
	DESTRUCTOR PUBLIC AuthCP ( ):

	END DESTRUCTOR.
	
	METHOD PUBLIC RAW getPBKey():
	    RETURN rEncryptKey.
	END.

END CLASS.
